#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <time.h>

#define DEBUG 0

int main (int argc,  char *argv[])
{
	char sourceLine[1024], processLine[1024], *charPtr;
	char nasPath[128]= ".", currentPath[8], computerMd5[50], computerFileName[128], tempString[128];
	char input[100] = ".", dest[100] = ".";
	FILE *fd;
	int x, y, z, localFileCount = 0, scriptFileCount = 0, successFileCount = 0, winB = 0, osxB = 0, nasB = 0, inputB = 0, destB = 0, matchB = 0;
	struct entry {
		char fileName[128], filePath[8], realFileName[128], md5[50];
		struct entry *next;
	};
	struct entry *entryCurrent = NULL, *entryList = NULL;
	time_t now;

//////////////////////////////////////////////////////////////////////
//   Arguement Management                                           //
//////////////////////////////////////////////////////////////////////

	if ((argc>1) && (DEBUG)) printf("Arguement: %s\n",argv[1]);
	if (argc==1) {
		fprintf(stderr,"Error:  No options entered.  Please refer to help using '-h' option.\n");
		return (1);
	}
	if ((strcmp(argv[1], "-h")==0) || (strcmp(argv[1], "--help"))==0) {
		fprintf(stderr,"\n\nalm-script2 (ATV Link Maker - Script 2) - Version 0.80\n");
		fprintf(stderr,"https://bitbucket.org/TexasEric/alm-script/overview\n\n");
		fprintf(stderr,"ASSUMPTIONS:\n");
		fprintf(stderr,"This program assumes you have: (1) run script1 and synced the results from your PC to this ATV, (2) mounted path to your NAS files, and (3) copied the md5_dat file that was produced from script1 onto the ATV.\n\n");
		fprintf(stderr,"NOTE:\n");
		fprintf(stderr,"This program does NOT make the actual changes to the files.  This program outputs a shell script that you can use to make the changes.\n\n");
		fprintf(stderr,"Usage: alm-scrpit2 OPTION...\n");
		fprintf(stderr,"  -h, --help        Present this help information\n");
		fprintf(stderr,"  -w, --win         MD5 source is from the cygwin version of script1\n");
		fprintf(stderr,"  -o, --osx         MD5 source is from the OSX version of script1\n");
		fprintf(stderr,"  -n, --nas         FULL Path to actual NAS media files.  Default is current location\n");
		fprintf(stderr,"  -i, --input       Path to md5_data file.  Default is current location\n");
		fprintf(stderr,"  -d, --dest        Path for resulting files.  Default is current location\n");
		fprintf(stderr,"\n\nExamples: ./alm-script2 -o -n /Users/frontrow/Movies\n\n");
		return(0);
	}

	for (x=1; x<argc; x++)
	{
		if (DEBUG) fprintf(stderr,"Arguement %i: %s\n",x,argv[x]);
		if ((strcmp(argv[x], "-w")==0) || (strcmp(argv[x], "--win")==0)) {
			winB = 1;
		}
		if ((strcmp(argv[x], "-o")==0) || (strcmp(argv[x], "--osx")==0)) {
			osxB = 1;
		}
		if ((strcmp(argv[x], "-i")==0) || (strcmp(argv[x], "--input")==0)) {
			inputB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') input[z++]=argv[x+1][y];
				else {
					input[z++]='\\';
					input[z++]=' ';
				}
			input[z]='\0';
			sprintf(processLine,"%s/md5_data",input);
			if((fd = fopen(processLine, "rb"))==NULL) {
				printf("Error: Cannot open md5_data file [%s].\n",processLine);
    				exit(1);
			}
			fclose(fd);

		}
		if ((strcmp(argv[x], "-n")==0) || (strcmp(argv[x], "--nas")==0)) {
			nasB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') nasPath[z++]=argv[x+1][y];
				else {
					nasPath[z++]='\\';
					nasPath[z++]=' ';
				}
			nasPath[z]='\0';
		}
		if ((strcmp(argv[x], "-d")==0) || (strcmp(argv[x], "--dest")==0)) {
			destB = 1;
			y=0;
			z=0;
			//if argument has a space in it, put a \ in front.
			for (y=0;y<strlen (argv[x+1]);y++)
				if (argv[x+1][y]!= ' ') dest[z++]=argv[x+1][y];
				else {
					dest[z++]='\\';
					dest[z++]=' ';
				}
			dest[z++]='/';
			dest[z]='\0';
		}
	}
	if ((winB + osxB)!=1) {
		fprintf(stderr,"Error: md5_data source details unknown.  Was script1 run in cygwin (-w) or OSX (-o)?\n");
		exit(1);
	}

//////////////////////////////////////////////////////////////////////
//   Pre-processing file management                                 //
//////////////////////////////////////////////////////////////////////

	// make output directory (if it does not exist)
	fprintf(stderr, "Making destination folder:  %s\n", dest);
	mkdir (dest, S_IRWXG | S_IRWXO | S_IRWXU);

	// remove old processing files (if they exist)
	sprintf(processLine,"%s/alm-script3",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	sprintf(processLine,"%s/localfiles",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	sprintf(processLine,"%s/localmd5.txt",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);

//////////////////////////////////////////////////////////////////////
//   Get local file list for processing                             //
//////////////////////////////////////////////////////////////////////

	sprintf (processLine,"ls -1R /mnt/Media/Media\\ Files/ >> %s/localfiles",dest);
	system(processLine);
	sprintf (processLine,"%s/localfiles",dest);
	fd = fopen(processLine,"r");

	fprintf(stderr, "Processing");
	while (fgets(sourceLine,1024,fd) != NULL) {
	//	if (DEBUG==1) fprintf(stderr, "CurrentLine: [%s]\n", sourceLine);
		if (sourceLine[strlen(sourceLine)-2] ==':')
		{
			memset (currentPath,'\0',sizeof (currentPath));
			strncpy(currentPath,strrchr(sourceLine,'/')+1,8);
			currentPath[strlen(currentPath)-2]='\0';
			fprintf(stderr, ".");
		}
		else if ((strstr(sourceLine, ".mp4") != NULL) || (strstr(sourceLine, ".m4v") != NULL)) {
			// Line is an mp4 or m4v file. Get the details.
			if (entryCurrent != NULL) {
				entryCurrent->next = (struct entry *) malloc(sizeof (struct entry));
				entryCurrent = entryCurrent->next;
			}
			else entryCurrent = (struct entry *) malloc(sizeof (struct entry));
			if (entryList == NULL) entryList = entryCurrent;
			entryCurrent->next = NULL;

			// fill fileName and filePath				
			strcpy(entryCurrent->fileName,sourceLine);
			entryCurrent->fileName[strlen(entryCurrent->fileName)-1] = '\0';
			strcpy(entryCurrent->filePath,currentPath);	
			strcpy(entryCurrent->realFileName,"Undetermined");

			localFileCount = localFileCount + 1;
			//Add md5 to local list
			sprintf(processLine,"md5 /mnt/Media/Media\\ Files/%s/%s >> %s/localmd5.txt",entryCurrent->filePath, entryCurrent->fileName, dest);
			system (processLine);
	//		if (DEBUG) fprintf(stderr, "Just read(%i): [%s] at [%s]\n", localFileCount,entryCurrent->fileName, entryCurrent->filePath);
		}
	} // while statement to read data
	fclose(fd);
	fprintf(stderr,"\n");

	// Load ATV Calulated Md5s
	sprintf (processLine,"%s/localmd5.txt",dest);
	fd = fopen(processLine,"r");
	while (fgets(sourceLine,1024,fd) != NULL) {
		entryCurrent=entryList;
		while  (entryCurrent != NULL) {
			strcpy(tempString,strrchr(sourceLine,')')-8);
			if (strncmp(tempString,entryCurrent->fileName,8) == 0 ) strncpy(entryCurrent->md5,strrchr(sourceLine,'=')+2,32);
			entryCurrent=entryCurrent->next;
		}
	} 
	fclose (fd);
	sprintf(processLine,"%s/localmd5.txt",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	//remove(processLine);

	fprintf (stderr, "\nFiles identified for processing on Apple TV: %i.\n",localFileCount);
	fclose(fd);

//////////////////////////////////////////////////////////////////////
//   Read md5 data from script1 and match ATV file to real file     //
//////////////////////////////////////////////////////////////////////

	// Load ATV Calulated Md5s
	sprintf (processLine,"%s/md5_data",input);
	if (DEBUG) fprintf(stderr, "Computer md5 Line: [%s]\n", processLine);
	fd = fopen(processLine,"r");
	while (fgets(sourceLine,1024,fd) != NULL) {
		if (DEBUG) fprintf(stderr, "CurrentLine: [%s]\n", sourceLine);
		if (winB == 1) {
			if (strrchr(sourceLine,'*') == NULL) {
				fprintf (stderr,"First line of md5_data does not look like cygwin md5 data.  Maybe you need the -o option, not -w.\n");
				exit (-1);
			}
			memset (computerMd5,'\0',sizeof (computerMd5));
			strncpy(computerMd5,sourceLine,32);
			strcpy(tempString, strrchr(sourceLine,'*')+1);
			if (strrchr(tempString,'/') != NULL) strcpy(tempString, strrchr(tempString,'/')+1);
			y = 0;
			z = 0;
			for (y=0;y<strlen(tempString);y++) {
				if ((tempString[y] == ' ') || (tempString[y] == '&')) computerFileName[z++] = '\\';
				computerFileName[z++] = tempString[y];
			}
			computerFileName[z-1] = '\0';
			if (DEBUG) fprintf (stderr, "Data from md5_data: [%s][%s][%s]\n",computerMd5,computerFileName,sourceLine);
		}
		else if (osxB == 1) {
			if (strrchr(sourceLine,'=') == NULL) {
				fprintf (stderr,"First line of md5_data does not look like OSX md5 data.  Maybe you need the -w option, not -o.\n");
				exit (-1);
			}
			memset (computerMd5,'\0',sizeof (computerMd5));
			strncpy(computerMd5,strrchr(sourceLine,'=')+2,32);
			strcpy(tempString, strrchr(sourceLine,'(')+1);
			if (strrchr(tempString,'/') != NULL) strcpy(tempString, strrchr(tempString,'/')+1);
			charPtr = strrchr(tempString,')');
			charPtr[0] = '\0';
			y = 0;
			z = 0;
			for (y=0;y<strlen(tempString);y++) {
				if ((tempString[y] == ' ') || (tempString[y] == '&')) computerFileName[z++] = '\\';
				computerFileName[z++] = tempString[y];
			}
			computerFileName[z] = '\0';
			if (DEBUG) fprintf (stderr, "Data from md5_data: [%s][%s][%s]\n",computerMd5,computerFileName,sourceLine);
		}
		entryCurrent = entryList;
		if (DEBUG) fprintf (stderr, "Data from md5_data to match: [%s][%s]\n",computerMd5,computerFileName);
		matchB = 0;
		while (entryCurrent != NULL) {
			if (DEBUG) fprintf (stderr, "    Attempted Match: [%s][%s]\n",entryCurrent->md5,entryCurrent->fileName);
			if (strncmp(entryCurrent->md5,computerMd5,32) != 0) entryCurrent=entryCurrent->next;
			else {
				strcpy (entryCurrent->realFileName, computerFileName);
				entryCurrent = NULL;
				matchB = 1;
				successFileCount++;
			}
		}
		if (matchB == 0) fprintf (stderr, "  Failed to match blank video: %s | %s\n", computerFileName,computerMd5);
	} // while statement to read md5s
	if (DEBUG) fprintf(stderr, "source Line: [%s]\n", sourceLine);
	fclose (fd);

	entryCurrent=entryList;
	while (entryCurrent != NULL) {
		if (strcmp(entryCurrent->realFileName,"Undetermined") == 0) {
			fprintf (stderr, "  Failed to match Apple TV file: %s/%s | %s\n", entryCurrent->filePath, entryCurrent->fileName, entryCurrent->md5);
		}
		entryCurrent = entryCurrent->next;
	}

	sprintf(processLine,"%s/localmd5.txt",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	fprintf (stderr, "Files matched from Apple TV and alm-script1: %i.\n",successFileCount);

	if (DEBUG) fprintf(stderr, "Ready to read data into memory: %s\n", processLine);


//////////////////////////////////////////////////////////////////////
//   Make Files!!                                                   //
//////////////////////////////////////////////////////////////////////

	fprintf (stderr, "Making 'alm-script3' file\n");
	entryCurrent = entryList;
	scriptFileCount = 0;
	sprintf(processLine, "echo '# alm-script3 auto-generated by alm-script2' >> %s/alm-script3",dest);
	system(processLine);
	sprintf(processLine,"echo '# https://bitbucket.org/TexasEric/alm-script/overview' >> %s/alm-script3",dest);
	system(processLine);
	time (&now);
	sprintf(tempString,"%s",ctime(&now));
	tempString[strlen(tempString)-1] = '\0';
	sprintf(processLine, "echo '# creation date: %s' >> %s/alm-script3",tempString,dest);
	system(processLine);
	sprintf(processLine, "echo '#!/bin/sh' >> %s/alm-script3",dest);
	system(processLine);
	while (entryCurrent != NULL) {
		sprintf(processLine, "echo 'rm /mnt/Media/Media\\ Files/%s/%s' >> %s/alm-script3",entryCurrent->filePath,entryCurrent->fileName,dest);
		system(processLine);
		sprintf(processLine, "echo 'ln -s %s/%s /mnt/Media/Media\\ Files/%s/%s' >> %s/alm-script3",nasPath,entryCurrent->realFileName,entryCurrent->filePath,entryCurrent->fileName,dest);
		system(processLine);
		
		scriptFileCount++;
		entryCurrent = entryCurrent->next;
	}
	fprintf (stderr, "Lines written to alm-script3: %i.\n\n",scriptFileCount);

//////////////////////////////////////////////////////////////////////
//   Post-processing clean-up                                       //
//////////////////////////////////////////////////////////////////////

	// remove old processing files (if they does exist)
	sprintf(processLine,"%s/localfiles",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);
	sprintf(processLine,"%s/filelist",dest);
	if (DEBUG) fprintf(stderr, "Removing contents of %s\n", processLine);
	remove(processLine);

	sprintf(processLine, "%s/alm-script3",dest);
	chmod(processLine, S_IRWXG | S_IRWXO | S_IRWXU);

	fprintf (stderr,"----------------------------------------------------------\nAll Done.  Run the resulting script3 file: './alm-script3' to make the links.  Note that file is a text file and easily edited and viewed.\n\n",successFileCount,dest);
}
